@@ -24,10 +24,17 @@
 #define XEMBED_MAPPED (1 << 0)
 #define XEMBED_EMBEDDED_NOTIFY 0
 
+/* We define xcb_request_failed as a macro to include the relevant line number */
+#define xcb_request_failed(cookie, err_msg) _xcb_request_failed(cookie, err_msg, __LINE__)
+int _xcb_request_failed(xcb_void_cookie_t cookie, char *err_msg, int line);
+
 struct xcb_color_strings_t {
     char *bar_fg;
     char *bar_bg;
     char *sep_fg;
+    char *focus_bar_fg;
+    char *focus_bar_bg;
+    char *focus_sep_fg;
     char *active_ws_fg;
     char *active_ws_bg;
     char *active_ws_border;
