@@ -75,6 +75,8 @@ static void clear_statusline(struct statusline_head *head, bool free_resources)
             FREE(first->name);
             FREE(first->instance);
             FREE(first->min_width_str);
+            FREE(first->background);
+            FREE(first->border);
         }
 
         TAILQ_REMOVE(head, first, blocks);
@@ -205,8 +207,16 @@ static int stdin_string(void *context, const unsigned char *val, size_t len) {
         sasprintf(&(ctx->block.color), "%.*s", len, val);
         return 1;
     }
+    if (strcasecmp(ctx->last_map_key, "background") == 0) {
+        sasprintf(&(ctx->block.background), "%.*s", len, val);
+        return 1;
+    }
+    if (strcasecmp(ctx->last_map_key, "border") == 0) {
+        sasprintf(&(ctx->block.border), "%.*s", len, val);
+        return 1;
+    }
     if (strcasecmp(ctx->last_map_key, "markup") == 0) {
-        ctx->block.is_markup = (len == strlen("pango") && !strncasecmp((const char *)val, "pango", strlen("pango")));
+        ctx->block.pango_markup = (len == strlen("pango") && !strncasecmp((const char *)val, "pango", strlen("pango")));
         return 1;
     }
     if (strcasecmp(ctx->last_map_key, "align") == 0) {
@@ -220,24 +230,15 @@ static int stdin_string(void *context, const unsigned char *val, size_t len) {
         return 1;
     }
     if (strcasecmp(ctx->last_map_key, "min_width") == 0) {
-        char *copy = (char *)smalloc(len + 1);
-        strncpy(copy, (const char *)val, len);
-        copy[len] = 0;
-        ctx->block.min_width_str = copy;
+        sasprintf(&(ctx->block.min_width_str), "%.*s", len, val);
         return 1;
     }
     if (strcasecmp(ctx->last_map_key, "name") == 0) {
-        char *copy = (char *)smalloc(len + 1);
-        strncpy(copy, (const char *)val, len);
-        copy[len] = 0;
-        ctx->block.name = copy;
+        sasprintf(&(ctx->block.name), "%.*s", len, val);
         return 1;
     }
     if (strcasecmp(ctx->last_map_key, "instance") == 0) {
-        char *copy = (char *)smalloc(len + 1);
-        strncpy(copy, (const char *)val, len);
-        copy[len] = 0;
-        ctx->block.instance = copy;
+        sasprintf(&(ctx->block.instance), "%.*s", len, val);
         return 1;
     }
 
@@ -275,15 +276,15 @@ static int stdin_end_map(void *context) {
 
     if (new_block->min_width_str) {
         i3String *text = i3string_from_utf8(new_block->min_width_str);
-        i3string_set_markup(text, new_block->is_markup);
+        i3string_set_markup(text, new_block->pango_markup);
         new_block->min_width = (uint32_t)predict_text_width(text);
         i3string_free(text);
     }
 
-    i3string_set_markup(new_block->full_text, new_block->is_markup);
+    i3string_set_markup(new_block->full_text, new_block->pango_markup);
 
     if (new_block->short_text != NULL)
-        i3string_set_markup(new_block->short_text, new_block->is_markup);
+        i3string_set_markup(new_block->short_text, new_block->pango_markup);
 
     TAILQ_INSERT_TAIL(&statusline_buffer, new_block, blocks);
     return 1;
