@@ -101,53 +101,39 @@ void x_con_init(Con *con, uint16_t depth) {
     uint32_t mask = 0;
     uint32_t values[5];
 
-    xcb_visualid_t visual = XCB_COPY_FROM_PARENT;
-    xcb_colormap_t win_colormap = XCB_NONE;
-    if (depth != root_depth && depth != XCB_COPY_FROM_PARENT) {
-        /* For custom visuals, we need to create a colormap before creating
-         * this window. It will be freed directly after creating the window. */
-        visual = get_visualid_by_depth(depth);
-        win_colormap = xcb_generate_id(conn);
-        xcb_create_colormap_checked(conn, XCB_COLORMAP_ALLOC_NONE, win_colormap, root, visual);
+    /* For custom visuals, we need to create a colormap before creating
+     * this window. It will be freed directly after creating the window. */
+    xcb_visualid_t visual = get_visualid_by_depth(depth);
+    xcb_colormap_t win_colormap = xcb_generate_id(conn);
+    xcb_create_colormap_checked(conn, XCB_COLORMAP_ALLOC_NONE, win_colormap, root, visual);
 
-        /* We explicitly set a background color and border color (even though we
-         * don’t even have a border) because the X11 server requires us to when
-         * using 32 bit color depths, see
-         * http://stackoverflow.com/questions/3645632 */
-        mask |= XCB_CW_BACK_PIXEL;
-        values[0] = root_screen->black_pixel;
+    /* We explicitly set a background color and border color (even though we
+     * don’t even have a border) because the X11 server requires us to when
+     * using 32 bit color depths, see
+     * http://stackoverflow.com/questions/3645632 */
+    mask |= XCB_CW_BACK_PIXEL;
+    values[0] = root_screen->black_pixel;
 
-        mask |= XCB_CW_BORDER_PIXEL;
-        values[1] = root_screen->black_pixel;
+    mask |= XCB_CW_BORDER_PIXEL;
+    values[1] = root_screen->black_pixel;
 
-        /* our own frames should not be managed */
-        mask |= XCB_CW_OVERRIDE_REDIRECT;
-        values[2] = 1;
+    /* our own frames should not be managed */
+    mask |= XCB_CW_OVERRIDE_REDIRECT;
+    values[2] = 1;
 
-        /* see include/xcb.h for the FRAME_EVENT_MASK */
-        mask |= XCB_CW_EVENT_MASK;
-        values[3] = FRAME_EVENT_MASK & ~XCB_EVENT_MASK_ENTER_WINDOW;
+    /* see include/xcb.h for the FRAME_EVENT_MASK */
+    mask |= XCB_CW_EVENT_MASK;
+    values[3] = FRAME_EVENT_MASK & ~XCB_EVENT_MASK_ENTER_WINDOW;
 
-        mask |= XCB_CW_COLORMAP;
-        values[4] = win_colormap;
-    } else {
-        /* our own frames should not be managed */
-        mask = XCB_CW_OVERRIDE_REDIRECT;
-        values[0] = 1;
-
-        /* see include/xcb.h for the FRAME_EVENT_MASK */
-        mask |= XCB_CW_EVENT_MASK;
-        values[1] = FRAME_EVENT_MASK & ~XCB_EVENT_MASK_ENTER_WINDOW;
-
-        mask |= XCB_CW_COLORMAP;
-        values[2] = colormap;
-    }
+    mask |= XCB_CW_COLORMAP;
+    values[4] = win_colormap;
 
     Rect dims = {-15, -15, 10, 10};
-    con->frame = create_window(conn, dims, depth, visual, XCB_WINDOW_CLASS_INPUT_OUTPUT, XCURSOR_CURSOR_POINTER, false, mask, values);
+    xcb_window_t frame_id = create_window(conn, dims, depth, visual, XCB_WINDOW_CLASS_INPUT_OUTPUT, XCURSOR_CURSOR_POINTER, false, mask, values);
+    draw_util_surface_init(conn, &(con->frame), frame_id, get_visualtype_by_id(visual), dims.width, dims.height);
     xcb_change_property(conn,
                         XCB_PROP_MODE_REPLACE,
-                        con->frame,
+                        con->frame.id,
                         XCB_ATOM_WM_CLASS,
                         XCB_ATOM_STRING,
                         8,
@@ -158,7 +144,7 @@ void x_con_init(Con *con, uint16_t depth) {
         xcb_free_colormap(conn, win_colormap);
 
     struct con_state *state = scalloc(1, sizeof(struct con_state));
-    state->id = con->frame;
+    state->id = con->frame.id;
     state->mapped = false;
     state->initial = true;
     DLOG("Adding window 0x%08x to lists\n", state->id);
@@ -177,7 +163,7 @@ void x_con_init(Con *con, uint16_t depth) {
 void x_reinit(Con *con) {
     struct con_state *state;
 
-    if ((state = state_for_frame(con->frame)) == NULL) {
+    if ((state = state_for_frame(con->frame.id)) == NULL) {
         ELOG("window state not found\n");
         return;
     }
@@ -196,13 +182,13 @@ void x_reinit(Con *con) {
  */
 void x_reparent_child(Con *con, Con *old) {
     struct con_state *state;
-    if ((state = state_for_frame(con->frame)) == NULL) {
+    if ((state = state_for_frame(con->frame.id)) == NULL) {
         ELOG("window state for con not found\n");
         return;
     }
 
     state->need_reparent = true;
-    state->old_frame = old->frame;
+    state->old_frame = old->frame.id;
 }
 
 /*
@@ -212,12 +198,12 @@ void x_reparent_child(Con *con, Con *old) {
 void x_move_win(Con *src, Con *dest) {
     struct con_state *state_src, *state_dest;
 
-    if ((state_src = state_for_frame(src->frame)) == NULL) {
+    if ((state_src = state_for_frame(src->frame.id)) == NULL) {
         ELOG("window state for src not found\n");
         return;
     }
 
-    if ((state_dest = state_for_frame(dest->frame)) == NULL) {
+    if ((state_dest = state_for_frame(dest->frame.id)) == NULL) {
         ELOG("window state for dest not found\n");
         return;
     }
@@ -239,10 +225,11 @@ void x_move_win(Con *src, Con *dest) {
 void x_con_kill(Con *con) {
     con_state *state;
 
-    xcb_destroy_window(conn, con->frame);
-    xcb_free_pixmap(conn, con->pixmap);
-    xcb_free_gc(conn, con->pm_gc);
-    state = state_for_frame(con->frame);
+    draw_util_surface_free(conn, &(con->frame));
+    draw_util_surface_free(conn, &(con->frame_buffer));
+    xcb_destroy_window(conn, con->frame.id);
+    xcb_free_pixmap(conn, con->frame_buffer.id);
+    state = state_for_frame(con->frame.id);
     CIRCLEQ_REMOVE(&state_head, state, state);
     CIRCLEQ_REMOVE(&old_state_head, state, old_state);
     TAILQ_REMOVE(&initial_mapping_head, state, initial_mapping_order);
@@ -312,6 +299,54 @@ void x_window_kill(xcb_window_t window, kill_window_t kill_window) {
     free(event);
 }
 
+static void x_draw_title_border(Con *con, struct deco_render_params *p) {
+    assert(con->parent != NULL);
+
+    Rect *dr = &(con->deco_rect);
+    adjacent_t borders_to_hide = con_adjacent_borders(con) & config.hide_edge_borders;
+    int deco_diff_l = borders_to_hide & ADJ_LEFT_SCREEN_EDGE ? 0 : con->current_border_width;
+    int deco_diff_r = borders_to_hide & ADJ_RIGHT_SCREEN_EDGE ? 0 : con->current_border_width;
+    if (con->parent->layout == L_TABBED ||
+        (con->parent->layout == L_STACKED && TAILQ_NEXT(con, nodes) != NULL)) {
+        deco_diff_l = 0;
+        deco_diff_r = 0;
+    }
+
+    draw_util_rectangle(conn, &(con->parent->frame_buffer), p->color->border,
+                        dr->x, dr->y, dr->width, 1);
+
+    draw_util_rectangle(conn, &(con->parent->frame_buffer), p->color->border,
+                        dr->x + deco_diff_l, dr->y + dr->height - 1, dr->width - (deco_diff_l + deco_diff_r), 1);
+}
+
+static void x_draw_decoration_after_title(Con *con, struct deco_render_params *p) {
+    assert(con->parent != NULL);
+
+    Rect *dr = &(con->deco_rect);
+    Rect br = con_border_style_rect(con);
+
+    /* Redraw the right border to cut off any text that went past it.
+     * This is necessary when the text was drawn using XCB since cutting text off
+     * automatically does not work there. For pango rendering, this isn't necessary. */
+    draw_util_rectangle(conn, &(con->parent->frame_buffer), p->color->background,
+                        dr->x + dr->width + br.width, dr->y, -br.width, dr->height);
+
+    /* Draw a 1px separator line before and after every tab, so that tabs can
+     * be easily distinguished. */
+    if (con->parent->layout == L_TABBED) {
+        /* Left side */
+        draw_util_rectangle(conn, &(con->parent->frame_buffer), p->color->border,
+                            dr->x, dr->y, 1, dr->height);
+
+        /* Right side */
+        draw_util_rectangle(conn, &(con->parent->frame_buffer), p->color->border,
+                            dr->x + dr->width - 1, dr->y, 1, dr->height);
+    }
+
+    /* Redraw the border. */
+    x_draw_title_border(con, p);
+}
+
 /*
  * Draws the decoration of the given container onto its parent.
  *
@@ -343,7 +378,7 @@ void x_draw_decoration(Con *con) {
     /* Skip containers whose pixmap has not yet been created (can happen when
      * decoration rendering happens recursively for a window for which
      * x_push_node() was not yet called) */
-    if (leaf && con->pixmap == XCB_NONE)
+    if (leaf && con->frame_buffer.id == XCB_NONE)
         return;
 
     /* 1: build deco_params and compare with cache */
@@ -395,29 +430,20 @@ void x_draw_decoration(Con *con) {
     con->pixmap_recreated = false;
     con->mark_changed = false;
 
-    /* 2: draw the client.background, but only for the parts around the client_rect */
+    /* 2: draw the client.background, but only for the parts around the window_rect */
     if (con->window != NULL) {
-        xcb_rectangle_t background[] = {
-            /* top area */
-            {0, 0, r->width, w->y},
-            /* bottom area */
-            {0, (w->y + w->height), r->width, r->height - (w->y + w->height)},
-            /* left area */
-            {0, 0, w->x, r->height},
-            /* right area */
-            {w->x + w->width, 0, r->width - (w->x + w->width), r->height}};
-#if 0
-        for (int i = 0; i < 4; i++)
-            DLOG("rect is (%d, %d) with %d x %d\n",
-                    background[i].x,
-                    background[i].y,
-                    background[i].width,
-                    background[i].height
-                );
-#endif
-
-        xcb_change_gc(conn, con->pm_gc, XCB_GC_FOREGROUND, (uint32_t[]){config.client.background});
-        xcb_poly_fill_rectangle(conn, con->pixmap, con->pm_gc, sizeof(background) / sizeof(xcb_rectangle_t), background);
+        /* top area */
+        draw_util_rectangle(conn, &(con->frame_buffer), config.client.background,
+                            0, 0, r->width, w->y);
+        /* bottom area */
+        draw_util_rectangle(conn, &(con->frame_buffer), config.client.background,
+                            0, w->y + w->height, r->width, r->height - (w->y + w->height));
+        /* left area */
+        draw_util_rectangle(conn, &(con->frame_buffer), config.client.background,
+                            0, 0, w->x, r->height);
+        /* right area */
+        draw_util_rectangle(conn, &(con->frame_buffer), config.client.background,
+                            w->x + w->width, 0, r->width - (w->x + w->width), r->height);
     }
 
     /* 3: draw a rectangle in border color around the client */
@@ -433,27 +459,27 @@ void x_draw_decoration(Con *con) {
         DLOG("window_rect spans (%d, %d) with %d x %d\n", con->window_rect.x, con->window_rect.y, con->window_rect.width, con->window_rect.height);
 #endif
 
-        /* These rectangles represents the border around the child window
+        /* These rectangles represent the border around the child window
          * (left, bottom and right part). We don’t just fill the whole
          * rectangle because some childs are not freely resizable and we want
          * their background color to "shine through". */
-        xcb_change_gc(conn, con->pm_gc, XCB_GC_FOREGROUND, (uint32_t[]){p->color->background});
         if (!(borders_to_hide & ADJ_LEFT_SCREEN_EDGE)) {
-            xcb_rectangle_t leftline = {0, 0, br.x, r->height};
-            xcb_poly_fill_rectangle(conn, con->pixmap, con->pm_gc, 1, &leftline);
+            draw_util_rectangle(conn, &(con->frame_buffer), p->color->child_border, 0, 0, br.x, r->height);
         }
         if (!(borders_to_hide & ADJ_RIGHT_SCREEN_EDGE)) {
-            xcb_rectangle_t rightline = {r->width + (br.width + br.x), 0, -(br.width + br.x), r->height};
-            xcb_poly_fill_rectangle(conn, con->pixmap, con->pm_gc, 1, &rightline);
+            draw_util_rectangle(conn, &(con->frame_buffer),
+                                p->color->child_border, r->width + (br.width + br.x), 0,
+                                -(br.width + br.x), r->height);
         }
         if (!(borders_to_hide & ADJ_LOWER_SCREEN_EDGE)) {
-            xcb_rectangle_t bottomline = {br.x, r->height + (br.height + br.y), r->width + br.width, -(br.height + br.y)};
-            xcb_poly_fill_rectangle(conn, con->pixmap, con->pm_gc, 1, &bottomline);
+            draw_util_rectangle(conn, &(con->frame_buffer),
+                                p->color->child_border, br.x, r->height + (br.height + br.y),
+                                r->width + br.width, -(br.height + br.y));
         }
-        /* 1pixel border needs an additional line at the top */
+        /* pixel border needs an additional line at the top */
         if (p->border_style == BS_PIXEL && !(borders_to_hide & ADJ_UPPER_SCREEN_EDGE)) {
-            xcb_rectangle_t topline = {br.x, 0, r->width + br.width, br.y};
-            xcb_poly_fill_rectangle(conn, con->pixmap, con->pm_gc, 1, &topline);
+            draw_util_rectangle(conn, &(con->frame_buffer),
+                                p->color->child_border, br.x, 0, r->width + br.width, br.y);
         }
 
         /* Highlight the side of the border at which the next window will be
@@ -463,13 +489,13 @@ void x_draw_decoration(Con *con) {
         if (TAILQ_NEXT(con, nodes) == NULL &&
             TAILQ_PREV(con, nodes_head, nodes) == NULL &&
             con->parent->type != CT_FLOATING_CON) {
-            xcb_change_gc(conn, con->pm_gc, XCB_GC_FOREGROUND, (uint32_t[]){p->color->indicator});
-            if (p->parent_layout == L_SPLITH)
-                xcb_poly_fill_rectangle(conn, con->pixmap, con->pm_gc, 1, (xcb_rectangle_t[]){
-                                                                              {r->width + (br.width + br.x), br.y, -(br.width + br.x), r->height + br.height}});
-            else if (p->parent_layout == L_SPLITV)
-                xcb_poly_fill_rectangle(conn, con->pixmap, con->pm_gc, 1, (xcb_rectangle_t[]){
-                                                                              {br.x, r->height + (br.height + br.y), r->width + br.width, -(br.height + br.y)}});
+            if (p->parent_layout == L_SPLITH) {
+                draw_util_rectangle(conn, &(con->frame_buffer), p->color->indicator,
+                                    r->width + (br.width + br.x), br.y, -(br.width + br.x), r->height + br.height);
+            } else if (p->parent_layout == L_SPLITV) {
+                draw_util_rectangle(conn, &(con->frame_buffer), p->color->indicator,
+                                    br.x, r->height + (br.height + br.y), r->width + br.width, -(br.height + br.y));
+            }
         }
     }
 
@@ -478,48 +504,49 @@ void x_draw_decoration(Con *con) {
     if (p->border_style != BS_NORMAL)
         goto copy_pixmaps;
 
+    /* If the parent hasn't been set up yet, skip the decoration rendering
+     * for now. */
+    if (parent->frame_buffer.id == XCB_NONE)
+        goto copy_pixmaps;
+
+    /* For the first child, we clear the parent pixmap to ensure there's no
+     * garbage left on there. This is important to avoid tearing when using
+     * transparency. */
+    if (con == TAILQ_FIRST(&(con->parent->nodes_head))) {
+        draw_util_clear_surface(conn, &(con->parent->frame_buffer), COLOR_TRANSPARENT);
+        FREE(con->parent->deco_render_params);
+    }
+
     /* 4: paint the bar */
-    xcb_change_gc(conn, parent->pm_gc, XCB_GC_FOREGROUND, (uint32_t[]){p->color->background});
-    xcb_rectangle_t drect = {con->deco_rect.x, con->deco_rect.y, con->deco_rect.width, con->deco_rect.height};
-    xcb_poly_fill_rectangle(conn, parent->pixmap, parent->pm_gc, 1, &drect);
+    draw_util_rectangle(conn, &(parent->frame_buffer), p->color->background,
+                        con->deco_rect.x, con->deco_rect.y, con->deco_rect.width, con->deco_rect.height);
 
     /* 5: draw two unconnected horizontal lines in border color */
-    xcb_change_gc(conn, parent->pm_gc, XCB_GC_FOREGROUND, (uint32_t[]){p->color->border});
-    Rect *dr = &(con->deco_rect);
-    adjacent_t borders_to_hide = con_adjacent_borders(con) & config.hide_edge_borders;
-    int deco_diff_l = borders_to_hide & ADJ_LEFT_SCREEN_EDGE ? 0 : con->current_border_width;
-    int deco_diff_r = borders_to_hide & ADJ_RIGHT_SCREEN_EDGE ? 0 : con->current_border_width;
-    if (parent->layout == L_TABBED ||
-        (parent->layout == L_STACKED && TAILQ_NEXT(con, nodes) != NULL)) {
-        deco_diff_l = 0;
-        deco_diff_r = 0;
-    }
-    xcb_segment_t segments[] = {
-        {dr->x, dr->y,
-         dr->x + dr->width - 1, dr->y},
-        {dr->x + deco_diff_l, dr->y + dr->height - 1,
-         dr->x - deco_diff_r + dr->width - 1, dr->y + dr->height - 1}};
-    xcb_poly_segment(conn, parent->pixmap, parent->pm_gc, 2, segments);
+    x_draw_title_border(con, p);
 
     /* 6: draw the title */
-    set_font_colors(parent->pm_gc, p->color->text, p->color->background);
     int text_offset_y = (con->deco_rect.height - config.font.height) / 2;
 
     struct Window *win = con->window;
     if (win == NULL) {
-        /* we have a split container which gets a representation
-         * of its children as title
-         */
-        char *title;
-        char *tree = con_get_tree_representation(con);
-        sasprintf(&title, "i3: %s", tree);
-        free(tree);
+        i3String *title;
+        if (con->title_format == NULL) {
+            char *_title;
+            char *tree = con_get_tree_representation(con);
+            sasprintf(&_title, "i3: %s", tree);
+            free(tree);
 
-        draw_text_ascii(title,
-                        parent->pixmap, parent->pm_gc,
-                        con->deco_rect.x + 2, con->deco_rect.y + text_offset_y,
-                        con->deco_rect.width - 2);
-        free(title);
+            title = i3string_from_utf8(_title);
+            FREE(_title);
+        } else {
+            title = con_parse_title_format(con);
+        }
+
+        draw_util_text(title, &(parent->frame_buffer),
+                       p->color->text, p->color->background,
+                       con->deco_rect.x + 2, con->deco_rect.y + text_offset_y,
+                       con->deco_rect.width - 2);
+        I3STRING_FREE(title);
 
         goto after_title;
     }
@@ -545,56 +572,49 @@ void x_draw_decoration(Con *con) {
     int indent_px = (indent_level * 5) * indent_mult;
 
     int mark_width = 0;
-    if (config.show_marks && con->mark != NULL && (con->mark)[0] != '_') {
-        char *formatted_mark;
-        sasprintf(&formatted_mark, "[%s]", con->mark);
-        i3String *mark = i3string_from_utf8(formatted_mark);
+    if (config.show_marks && !TAILQ_EMPTY(&(con->marks_head))) {
+        char *formatted_mark = sstrdup("");
+        bool had_visible_mark = false;
+
+        mark_t *mark;
+        TAILQ_FOREACH(mark, &(con->marks_head), marks) {
+            if (mark->name[0] == '_')
+                continue;
+            had_visible_mark = true;
+
+            char *buf;
+            sasprintf(&buf, "%s[%s]", formatted_mark, mark->name);
+            free(formatted_mark);
+            formatted_mark = buf;
+        }
+
+        if (had_visible_mark) {
+            i3String *mark = i3string_from_utf8(formatted_mark);
+            mark_width = predict_text_width(mark);
+
+            draw_util_text(mark, &(parent->frame_buffer),
+                           p->color->text, p->color->background,
+                           con->deco_rect.x + con->deco_rect.width - mark_width - logical_px(2),
+                           con->deco_rect.y + text_offset_y, mark_width);
+
+            I3STRING_FREE(mark);
+        }
+
         FREE(formatted_mark);
-        mark_width = predict_text_width(mark);
-
-        draw_text(mark, parent->pixmap, parent->pm_gc,
-                  con->deco_rect.x + con->deco_rect.width - mark_width - logical_px(2),
-                  con->deco_rect.y + text_offset_y, mark_width);
-
-        I3STRING_FREE(mark);
     }
 
-    i3String *title = win->title_format == NULL ? win->name : window_parse_title_format(win);
-    draw_text(title,
-              parent->pixmap, parent->pm_gc,
-              con->deco_rect.x + logical_px(2) + indent_px, con->deco_rect.y + text_offset_y,
-              con->deco_rect.width - logical_px(2) - indent_px - mark_width - logical_px(2));
-    if (win->title_format != NULL)
+    i3String *title = con->title_format == NULL ? win->name : con_parse_title_format(con);
+    draw_util_text(title, &(parent->frame_buffer),
+                   p->color->text, p->color->background,
+                   con->deco_rect.x + logical_px(2) + indent_px, con->deco_rect.y + text_offset_y,
+                   con->deco_rect.width - logical_px(2) - indent_px - mark_width - logical_px(2));
+    if (con->title_format != NULL)
         I3STRING_FREE(title);
 
 after_title:
-    /* Since we don’t clip the text at all, it might in some cases be painted
-     * on the border pixels on the right side of a window. Therefore, we draw
-     * the right border again after rendering the text (and the unconnected
-     * lines in border color). */
-
-    /* Draw a 1px separator line before and after every tab, so that tabs can
-     * be easily distinguished. */
-    if (parent->layout == L_TABBED) {
-        xcb_change_gc(conn, parent->pm_gc, XCB_GC_FOREGROUND, (uint32_t[]){p->color->border});
-    } else {
-        xcb_change_gc(conn, parent->pm_gc, XCB_GC_FOREGROUND, (uint32_t[]){p->color->background});
-    }
-    xcb_poly_line(conn, XCB_COORD_MODE_ORIGIN, parent->pixmap, parent->pm_gc, 6,
-                  (xcb_point_t[]){
-                      {dr->x + dr->width, dr->y},
-                      {dr->x + dr->width, dr->y + dr->height},
-                      {dr->x + dr->width - 1, dr->y},
-                      {dr->x + dr->width - 1, dr->y + dr->height},
-                      {dr->x, dr->y + dr->height},
-                      {dr->x, dr->y},
-                  });
-
-    xcb_change_gc(conn, parent->pm_gc, XCB_GC_FOREGROUND, (uint32_t[]){p->color->border});
-    xcb_poly_segment(conn, parent->pixmap, parent->pm_gc, 2, segments);
-
+    x_draw_decoration_after_title(con, p);
 copy_pixmaps:
-    xcb_copy_area(conn, con->pixmap, con->frame, con->pm_gc, 0, 0, 0, 0, con->rect.width, con->rect.height);
+    draw_util_copy_surface(conn, &(con->frame_buffer), &(con->frame), 0, 0, 0, 0, con->rect.width, con->rect.height);
 }
 
 /*
@@ -607,7 +627,7 @@ void x_deco_recurse(Con *con) {
     Con *current;
     bool leaf = TAILQ_EMPTY(&(con->nodes_head)) &&
                 TAILQ_EMPTY(&(con->floating_head));
-    con_state *state = state_for_frame(con->frame);
+    con_state *state = state_for_frame(con->frame.id);
 
     if (!leaf) {
         TAILQ_FOREACH(current, &(con->nodes_head), nodes)
@@ -616,8 +636,9 @@ void x_deco_recurse(Con *con) {
         TAILQ_FOREACH(current, &(con->floating_head), floating_windows)
         x_deco_recurse(current);
 
-        if (state->mapped)
-            xcb_copy_area(conn, con->pixmap, con->frame, con->pm_gc, 0, 0, 0, 0, con->rect.width, con->rect.height);
+        if (state->mapped) {
+            draw_util_copy_surface(conn, &(con->frame_buffer), &(con->frame), 0, 0, 0, 0, con->rect.width, con->rect.height);
+        }
     }
 
     if ((con->type != CT_ROOT && con->type != CT_OUTPUT) &&
@@ -634,7 +655,7 @@ static void set_hidden_state(Con *con) {
         return;
     }
 
-    con_state *state = state_for_frame(con->frame);
+    con_state *state = state_for_frame(con->frame.id);
     bool should_be_hidden = con_is_hidden(con);
     if (should_be_hidden == state->is_hidden)
         return;
@@ -662,12 +683,12 @@ void x_push_node(Con *con) {
     Rect rect = con->rect;
 
     //DLOG("Pushing changes for node %p / %s\n", con, con->name);
-    state = state_for_frame(con->frame);
+    state = state_for_frame(con->frame.id);
 
     if (state->name != NULL) {
         DLOG("pushing name %s for con %p\n", state->name, con);
 
-        xcb_change_property(conn, XCB_PROP_MODE_REPLACE, con->frame,
+        xcb_change_property(conn, XCB_PROP_MODE_REPLACE, con->frame.id,
                             XCB_ATOM_WM_NAME, XCB_ATOM_STRING, 8, strlen(state->name), state->name);
         FREE(state->name);
     }
@@ -700,7 +721,7 @@ void x_push_node(Con *con) {
         xcb_change_window_attributes(conn, state->old_frame, XCB_CW_EVENT_MASK, values);
         xcb_change_window_attributes(conn, con->window->id, XCB_CW_EVENT_MASK, values);
 
-        xcb_reparent_window(conn, con->window->id, con->frame, 0, 0);
+        xcb_reparent_window(conn, con->window->id, con->frame.id, 0, 0);
 
         values[0] = FRAME_EVENT_MASK;
         xcb_change_window_attributes(conn, state->old_frame, XCB_CW_EVENT_MASK, values);
@@ -722,11 +743,17 @@ void x_push_node(Con *con) {
                              con->parent->layout == L_STACKED ||
                              con->parent->layout == L_TABBED);
 
+    /* The root con and output cons will never require a pixmap. In particular for the
+     * __i3 output, this will likely not work anyway because it might be ridiculously
+     * large, causing an XCB_ALLOC error. */
+    if (con->type == CT_ROOT || con->type == CT_OUTPUT)
+        is_pixmap_needed = false;
+
     bool fake_notify = false;
     /* Set new position if rect changed (and if height > 0) or if the pixmap
      * needs to be recreated */
-    if ((is_pixmap_needed && con->pixmap == XCB_NONE) || (memcmp(&(state->rect), &rect, sizeof(Rect)) != 0 &&
-                                                          rect.height > 0)) {
+    if ((is_pixmap_needed && con->frame_buffer.id == XCB_NONE) || (memcmp(&(state->rect), &rect, sizeof(Rect)) != 0 &&
+                                                                   rect.height > 0)) {
         /* We first create the new pixmap, then render to it, set it as the
          * background and only afterwards change the window size. This reduces
          * flickering. */
@@ -739,38 +766,47 @@ void x_push_node(Con *con) {
 
         /* Check if the container has an unneeded pixmap left over from
          * previously having a border or titlebar. */
-        if (!is_pixmap_needed && con->pixmap != XCB_NONE) {
-            xcb_free_pixmap(conn, con->pixmap);
-            con->pixmap = XCB_NONE;
+        if (!is_pixmap_needed && con->frame_buffer.id != XCB_NONE) {
+            draw_util_surface_free(conn, &(con->frame_buffer));
+            xcb_free_pixmap(conn, con->frame_buffer.id);
+            con->frame_buffer.id = XCB_NONE;
         }
 
-        if (is_pixmap_needed && (has_rect_changed || con->pixmap == XCB_NONE)) {
-            if (con->pixmap == 0) {
-                con->pixmap = xcb_generate_id(conn);
-                con->pm_gc = xcb_generate_id(conn);
+        if (is_pixmap_needed && (has_rect_changed || con->frame_buffer.id == XCB_NONE)) {
+            if (con->frame_buffer.id == XCB_NONE) {
+                con->frame_buffer.id = xcb_generate_id(conn);
             } else {
-                xcb_free_pixmap(conn, con->pixmap);
-                xcb_free_gc(conn, con->pm_gc);
+                draw_util_surface_free(conn, &(con->frame_buffer));
+                xcb_free_pixmap(conn, con->frame_buffer.id);
             }
 
             uint16_t win_depth = root_depth;
             if (con->window)
                 win_depth = con->window->depth;
 
-            xcb_create_pixmap(conn, win_depth, con->pixmap, con->frame, rect.width, rect.height);
+            /* Ensure we have valid dimensions for our surface. */
+            // TODO This is probably a bug in the condition above as we should never enter this path
+            //      for height == 0. Also, we should probably handle width == 0 the same way.
+            int width = MAX((int32_t)rect.width, 1);
+            int height = MAX((int32_t)rect.height, 1);
+
+            xcb_create_pixmap_checked(conn, win_depth, con->frame_buffer.id, con->frame.id, width, height);
+            draw_util_surface_init(conn, &(con->frame_buffer), con->frame_buffer.id,
+                                   get_visualtype_by_id(get_visualid_by_depth(win_depth)), width, height);
 
             /* For the graphics context, we disable GraphicsExposure events.
              * Those will be sent when a CopyArea request cannot be fulfilled
              * properly due to parts of the source being unmapped or otherwise
              * unavailable. Since we always copy from pixmaps to windows, this
              * is not a concern for us. */
-            uint32_t values[] = {0};
-            xcb_create_gc(conn, con->pm_gc, con->pixmap, XCB_GC_GRAPHICS_EXPOSURES, values);
+            xcb_change_gc(conn, con->frame_buffer.gc, XCB_GC_GRAPHICS_EXPOSURES, (uint32_t[]){0});
 
+            draw_util_surface_set_size(&(con->frame), width, height);
             con->pixmap_recreated = true;
 
             /* Don’t render the decoration for windows inside a stack which are
              * not visible right now */
+            // TODO Should this work the same way for L_TABBED?
             if (!con->parent ||
                 con->parent->layout != L_STACKED ||
                 TAILQ_FIRST(&(con->parent->focus_head)) == con)
@@ -786,9 +822,10 @@ void x_push_node(Con *con) {
          * window get lost when resizing it, therefore we want to provide it as
          * fast as possible) */
         xcb_flush(conn);
-        xcb_set_window_rect(conn, con->frame, rect);
-        if (con->pixmap != XCB_NONE)
-            xcb_copy_area(conn, con->pixmap, con->frame, con->pm_gc, 0, 0, 0, 0, con->rect.width, con->rect.height);
+        xcb_set_window_rect(conn, con->frame.id, rect);
+        if (con->frame_buffer.id != XCB_NONE) {
+            draw_util_copy_surface(conn, &(con->frame_buffer), &(con->frame), 0, 0, 0, 0, con->rect.width, con->rect.height);
+        }
         xcb_flush(conn);
 
         memcpy(&(state->rect), &rect, sizeof(Rect));
@@ -832,17 +869,18 @@ void x_push_node(Con *con) {
             state->child_mapped = true;
         }
 
-        cookie = xcb_map_window(conn, con->frame);
+        cookie = xcb_map_window(conn, con->frame.id);
 
         values[0] = FRAME_EVENT_MASK;
-        xcb_change_window_attributes(conn, con->frame, XCB_CW_EVENT_MASK, values);
+        xcb_change_window_attributes(conn, con->frame.id, XCB_CW_EVENT_MASK, values);
 
         /* copy the pixmap contents to the frame window immediately after mapping */
-        if (con->pixmap != XCB_NONE)
-            xcb_copy_area(conn, con->pixmap, con->frame, con->pm_gc, 0, 0, 0, 0, con->rect.width, con->rect.height);
+        if (con->frame_buffer.id != XCB_NONE) {
+            draw_util_copy_surface(conn, &(con->frame_buffer), &(con->frame), 0, 0, 0, 0, con->rect.width, con->rect.height);
+        }
         xcb_flush(conn);
 
-        DLOG("mapping container %08x (serial %d)\n", con->frame, cookie.sequence);
+        DLOG("mapping container %08x (serial %d)\n", con->frame.id, cookie.sequence);
         state->mapped = con->mapped;
     }
 
@@ -876,7 +914,7 @@ static void x_push_node_unmaps(Con *con) {
     con_state *state;
 
     //DLOG("Pushing changes (with unmaps) for node %p / %s\n", con, con->name);
-    state = state_for_frame(con->frame);
+    state = state_for_frame(con->frame.id);
 
     /* map/unmap if map state changed, also ensure that the child window
      * is changed if we are mapped *and* in initial state (meaning the
@@ -890,14 +928,14 @@ static void x_push_node_unmaps(Con *con) {
                                 A_WM_STATE, A_WM_STATE, 32, 2, data);
         }
 
-        cookie = xcb_unmap_window(conn, con->frame);
+        cookie = xcb_unmap_window(conn, con->frame.id);
         DLOG("unmapping container %p / %s (serial %d)\n", con, con->name, cookie.sequence);
         /* we need to increase ignore_unmap for this container (if it
          * contains a window) and for every window "under" this one which
          * contains a window */
         if (con->window != NULL) {
             con->ignore_unmap++;
-            DLOG("ignore_unmap for con %p (frame 0x%08x) now %d\n", con, con->frame, con->ignore_unmap);
+            DLOG("ignore_unmap for con %p (frame 0x%08x) now %d\n", con, con->frame.id, con->ignore_unmap);
         }
         state->mapped = con->mapped;
     }
@@ -950,7 +988,10 @@ void x_push_changes(Con *con) {
 
     DLOG("-- PUSHING WINDOW STACK --\n");
     //DLOG("Disabling EnterNotify\n");
-    uint32_t values[1] = {XCB_NONE};
+    /* We need to keep SubstructureRedirect around, otherwise clients can send
+     * ConfigureWindow requests and get them applied directly instead of having
+     * them become ConfigureRequests that i3 handles. */
+    uint32_t values[1] = {XCB_EVENT_MASK_SUBSTRUCTURE_REDIRECT};
     CIRCLEQ_FOREACH_REVERSE(state, &state_head, state) {
         if (state->mapped)
             xcb_change_window_attributes(conn, state->id, XCB_CW_EVENT_MASK, values);
@@ -1037,6 +1078,8 @@ void x_push_changes(Con *con) {
                 xcb_warp_pointer(conn, XCB_NONE, root, 0, 0, 0, 0, mid_x, mid_y);
                 xcb_change_window_attributes(conn, root, XCB_CW_EVENT_MASK, (uint32_t[]){ROOT_EVENT_MASK});
             }
+
+            free(pointerreply);
         }
         warp_to = NULL;
     }
@@ -1051,7 +1094,7 @@ void x_push_changes(Con *con) {
 
     x_deco_recurse(con);
 
-    xcb_window_t to_focus = focused->frame;
+    xcb_window_t to_focus = focused->frame.id;
     if (focused->window != NULL)
         to_focus = focused->window->id;
 
@@ -1145,7 +1188,7 @@ void x_push_changes(Con *con) {
  */
 void x_raise_con(Con *con) {
     con_state *state;
-    state = state_for_frame(con->frame);
+    state = state_for_frame(con->frame.id);
     //DLOG("raising in new stack: %p / %s / %s / xid %08x\n", con, con->name, con->window ? con->window->name_json : "", state->id);
 
     CIRCLEQ_REMOVE(&state_head, state, state);
@@ -1161,7 +1204,7 @@ void x_raise_con(Con *con) {
 void x_set_name(Con *con, const char *name) {
     struct con_state *state;
 
-    if ((state = state_for_frame(con->frame)) == NULL) {
+    if ((state = state_for_frame(con->frame.id)) == NULL) {
         ELOG("window state not found\n");
         return;
     }
